/*
 * This document has been created by using VisualHAM
 * http://www.console-dev.de
 *
 * Author: Peter Schraut
 * Date..: 27th October 2003
 *
*/
#include <mygba.h>

// include hel header
#include <hel.h>

//
// Defines
//
#define MAPSCROLL_SPEED    3

//
// Global variables
//
u8  g_NewFrame=TRUE;
u8  g_Frames=0;

TMapScrollInfo MapInfo;
TPoint32 CurPos;

// extern graphics
extern const unsigned short isle_Map[262144];
extern const unsigned char isle_Tiles[36352];
extern const unsigned short master_Palette[256];

//
// Prototypes
//
void vblFunc(void);


void MapOnTopRowChanged(TMapScrollInfo *pMapInfo, u32 X, u32 Y)
{
   ham_DrawText(1, 1, "MapOnTopRowChanged    ");
   ham_DrawText(1, 2, "X: %d     ", X);
   ham_DrawText(1, 3, "Y: %d     ", Y);
}


void MapOnLeftColumnChanged(TMapScrollInfo *pMapInfo, u32 X, u32 Y)
{
   ham_DrawText(1, 1, "MapOnLeftColumnChanged");
   ham_DrawText(1, 2, "X: %d     ", X);
   ham_DrawText(1, 3, "Y: %d     ", Y);
}

/*********************************************************************************
 * Program entry point
 ********************************************************************************/
int main(void)
{
    // Initialize HAMlib
    // Then we call the single most important function of ham, ham_Init .
    // This function must be called first thing when working with HAMlib,
    // because all ham_* functions will only work correctly after this has been called.
    // Behaviour of HAMlib is undefined if you comment this out.
    ham_Init();
    
    // Sets the GBAs BGMode to the value specified in bgno
    ham_SetBgMode(0);


    // Init the Text display system on the bg of your choice
    // This function must be called if you want to use HAMlibs text display system.
    // After calling this, you can use ham_DrawText and ham_SetTextCol.
    ham_InitText(0);
    
    // Load Background Palette
    ham_LoadBGPal((void *)&master_Palette,         // a pointer to the palette data that is to be loaded
                  SIZEOF_16BIT(master_Palette));   // the size of the data in u16 chunks (usually 256)

    // Sets a RGB color value to the background palette
    TOOL_SET_BG_PAL_256(3, 255, 255, 255)    // B Value
    
    // Will set the color indices of the font to the BG colors specified
    ham_SetTextCol(0x03, 0x00);
    
    // This function will load a tileset (usually from ROM)
    // from the src pointer and copy it into an appropiate spot
    // in the GBAs BG VRAM. The spot is decided by the memory manager,
    // and passed back in the structure.
    ham_bg[2].ti = ham_InitTileSet((void*)&isle_Tiles,              // A pointer to the source data
                                    SIZEOF_16BIT(isle_Tiles),       // The size of the tiles to be copied into Tile RAM, in number of 16bit chunks
                                    1,                              // The Color mode of the tile set (0=16col 1=256col)
                                    1);                             // Setting this one requires a bit of understanding of the GBA hardware.
                                                                    // Basically, if you put tiles into the GBA memory, you will have to put
                                                                    // them at the start of a Character Base block in order to be able to
                                                                    // access tile #0 with a map entry of 0. If the allocated block is not
                                                                    // directly at the start of a CBB, you will have to offset your map entries.
                                                                    // This is automatically done for you when you use ham_SetMapTile.
                                                                    // However, when you load your map from ROM, you might want to align your
                                                                    // tileset to a CBB boundary. If in doubt, and you have some VRAM left, set
                                                                    // this to 1 always.
                                                                    // 0=allocate at the earliest possible spot, unsafe if you do not know / do not
                                                                    // want to offset your map entries!
                                                                    // 1=allocate only on a CBB start to ensure tile 0 is map index no 0.
                                                                    // Again, if you have no idea what this is, set it to 1.


    MapInfo = hel_MapInit(2,        // background
                          1024/8,   // mapwidth in tiles
                          1024/8,   // mapheight in tiles
                          FALSE,    // rotation map?
                          (void*)isle_Map);   // pointer to mapdata

    // setup callbacks
    hel_MapSetCallbacks(&MapInfo,
                        (PMapNotifyFunc)MapOnTopRowChanged,
                        (PMapNotifyFunc)MapOnLeftColumnChanged);  
    
    
    // You can call this function to actually commit this BG to the hardware.
    // This command will throw an error if the BG that you want to init has not yet
    // been assigned a ti and mi. For more information on how to assign these to a bg,
    //see the links below.
    ham_InitBg(2,
               1, 
               2,  
               1);
    
   // Will, once activated, trigger the specified interrupt,
   // and then call the function specified
   ham_StartIntHandler(INT_TYPE_VBL,         // The Interrupts ID you want to start.
                       (void *)&vblFunc);    // The adress of a function that should be called when the interrupt is fired

   hel_PadInit();
            
    // Loop
    while(1)
    {
        // New Frame?
        if(g_NewFrame)
        {
            // read current joypad state
            hel_PadReadState();
                      
            // Get joypad info and scroll around
            if(hel_Pad.Held.Down) 
            {
               hel_MapScrollBy(&MapInfo, 0, MAPSCROLL_SPEED);
               CurPos.Y+=MAPSCROLL_SPEED;
            }
            
            if(hel_Pad.Held.Up) 
            {
               hel_MapScrollBy(&MapInfo, 0, -MAPSCROLL_SPEED);
               if(CurPos.Y > 0) CurPos.Y-=MAPSCROLL_SPEED;
            }
            
            if(hel_Pad.Held.Left) 
            {
               hel_MapScrollBy(&MapInfo, -MAPSCROLL_SPEED, 0);
               if(CurPos.X > 0) CurPos.X-=MAPSCROLL_SPEED;
            }
            
            if(hel_Pad.Held.Right) 
            {
               hel_MapScrollBy(&MapInfo, MAPSCROLL_SPEED, 0);
               CurPos.X+=MAPSCROLL_SPEED;
            }
            
            if(hel_Pad.Pressed.A)
            {
               hel_MapSetPosition(&MapInfo, 10, 10);
               CurPos.X=hel_MapGetPositionInPixel(&MapInfo).X;
               CurPos.Y=hel_MapGetPositionInPixel(&MapInfo).Y;
            }
               
               
            // Show syncronization
            ham_DrawText(1, 17, "Map X,Y: %d, %d    ", hel_MapGetPositionInPixel(&MapInfo).X, hel_MapGetPositionInPixel(&MapInfo).Y);
            ham_DrawText(1, 18, "My  X,Y: %d, %d    ", CurPos.X, CurPos.Y);
               
            g_NewFrame=FALSE;
        }
    }
    
    hel_PadReset();
    hel_MapDeInit(&MapInfo);
}

/*********************************************************************************
 * vblFunc(void)
 *
 * This function is called whenever the GBA is about
 * to draw a new picture onto the screen.
 ********************************************************************************/
void vblFunc()
{
   ham_CopyObjToOAM();
   
	// Signify new frame
	g_NewFrame=TRUE;
	g_Frames++;
}

/* END OF FILE */
